/***************************************************************************

	File : getdata.c
	Author : Ran Nathan
	Date : October 24, 1999

	Overview:
	This file contains the implementation of the functions used in WINDSIPER
	to read the input data.
	Refer to file getdata.h for details and usage.

***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include <time.h>
#include "getdata.h"
#include "parse.h"

extern site_t *site;

extern int nodata;
extern unsigned long int xllcorner, yllcorner;
/*  Default variables required to the ARC/Info input/output files */
extern float Wind_Array[WIND_SECTORS][WIND_ARRAY_PARAMETERS];
/*  An array that stores directional wind data */

extern int Cell_Width;

extern int Rows;
extern int Columns;

extern BOOL GetGrid;
extern BOOL PeriodOut;
extern BOOL WindRandom;

extern float Mean_Tree_Height;
extern float Std_Tree_Height;
extern float Mean_Portion_Height;
extern float Std_Portion_Height;
extern float Max_Portion_Height;
extern float Min_Portion_Height;
extern float Mean_Terminal_Velocity;
extern float Std_Terminal_Velocity;
extern float Portion_Wingless;
extern float Threshold_Windspeed;
extern float Mean_Vertical_Windspeed;
extern float Std_Vertical_Windspeed;
extern float Site_Roughness;
extern float Site_Displacement;
extern float WS_Roughness;
extern float WS_Displacement;
extern float Measurement_Height;

extern char  Period_Input_File[1000];
extern char  Period_Output_File[1000];

int Get_All_Periods_Data(void)
{
	int number_of_periods;
	Record *list;
	char *result;

	list = Parse("windisp.dat", '=');

	if (list==NULL) {
		fprintf(stderr,"Parameter file open error");
		return 0;
	}

	if ( GetBOOL(list, "Tree distribution data is grid?", &GetGrid) == 0) {
		fprintf(stderr, "\nCannot find data on tree distribution data.	Program terminated.\n");
		DeleteRecord(list);
	  exit(1);  /* terminate program if critical data is missing */
	}

	if ( GetInt(list, "Number of periods", &number_of_periods) == 0) {
		number_of_periods = DEFAULT_NUMBER_OF_PERIODS;
		fprintf(stderr, "\nCannot find data on number of periods. Taking default.\n");
	}

	if ( (result=GetString(list, "Name of period input file")) == NULL) {
		fprintf(stderr, "\nCannot find data on period input file. Taking default.\n");
	}
	else
		strcpy(Period_Input_File, result);

	if ( (result=GetString(list, "Name of period output file")) == NULL) {
		fprintf(stderr, "\nCannot find data on period output file. Taking default.\n");
	}
	else
		strcpy(Period_Output_File, result);

	if ( GetBOOL(list, "Generate periodic output files?", &PeriodOut) == 0) {
		PeriodOut = FALSE;
		fprintf(stderr, "\nCannot find data on whether to generate period outputs. Taking default.\n");
	}

	if ( GetBOOL(list, "Wind is random?", &WindRandom) == 0) {
		WindRandom = TRUE;
		fprintf(stderr, "\nCannot find data on whether to simulate random winds. Taking default.\n");
	}

	if ( GetFloat(list, "Mean tree height", &Mean_Tree_Height) == 0) {
		Mean_Tree_Height = DEFAULT_MEAN_TREE_HEIGHT;
		fprintf(stderr, "\nCannot find data on mean tree height. Taking default\n");
	}

	if ( GetFloat(list, "Std tree height", &Std_Tree_Height) == 0) {
		Std_Tree_Height = DEFAULT_STD_TREE_HEIGHT;
		fprintf(stderr, "\nCannot find data on std tree height. Taking default\n");
	}

	if ( GetFloat(list, "Mean portion height", &Mean_Portion_Height) == 0) {
		Mean_Portion_Height = DEFAULT_MEAN_PORTION_HEIGHT;
		fprintf(stderr, "\nCannot find data on mean portion height. Taking default\n");
	}

	if ( GetFloat(list, "Std portion height", &Std_Portion_Height) == 0) {
		Std_Portion_Height = DEFAULT_STD_PORTION_HEIGHT;
		fprintf(stderr, "\nCannot find data on std portion height. Taking default\n");
	}

		if ( GetFloat(list, "Max portion height", &Max_Portion_Height) == 0) {
		Max_Portion_Height = DEFAULT_MAX_PORTION_HEIGHT;
		fprintf(stderr, "\nCannot find data on max portion height. Taking default\n");
	}

	if ( GetFloat(list, "Min portion height", &Min_Portion_Height) == 0) {
		Min_Portion_Height = DEFAULT_MIN_PORTION_HEIGHT;
		fprintf(stderr, "\nCannot find data on min portion height. Taking default\n");
	}

	if ( GetFloat(list, "Mean terminal velocity", &Mean_Terminal_Velocity) == 0) {
		Mean_Terminal_Velocity = DEFAULT_MEAN_TERMINAL_VELOCITY;
		fprintf(stderr, "\nCannot find data on mean terminal velocity. Taking default\n");
	}

	if ( GetFloat(list, "Std terminal velocity", &Std_Terminal_Velocity) == 0) {
		Std_Terminal_Velocity = DEFAULT_STD_TERMINAL_VELOCITY;
		fprintf(stderr, "\nCannot find data on std terminal velocity. Taking default\n");
	}

	if ( GetFloat(list, "Portion wingless", &Portion_Wingless) == 0) {
		Portion_Wingless = DEFAULT_PORTION_WINGLESS;
		fprintf(stderr, "\nCannot find data on portion wingless. Taking default\n");
	}

	if ( GetFloat(list, "Threshold windspeed", &Threshold_Windspeed) == 0) {
		Threshold_Windspeed = DEFAULT_THRESHOLD_WINDSPEED;
		fprintf(stderr, "\nCannot find data on threshold windspeed. Taking default\n");
	}

	if ( GetFloat(list, "Mean vertical windspeed", &Mean_Vertical_Windspeed) == 0) {
		Mean_Vertical_Windspeed = DEFAULT_MEAN_VERTICAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on mean vertical windspeed. Taking default\n");
	}

	if ( GetFloat(list, "Std vertical windspeed", &Std_Vertical_Windspeed) == 0) {
		Std_Vertical_Windspeed = DEFAULT_STD_VERTICAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on std vertical windspeed. Taking default\n");
	}

	if ( GetFloat(list, "Study site roughness", &Site_Roughness) == 0) {
		Site_Roughness = DEFAULT_SITE_ROUGHNESS;
		fprintf(stderr, "\nCannot find data on study site roughness. Taking default\n");
	}

	if ( GetFloat(list, "Study site displacement", &Site_Displacement) == 0) {
		Site_Displacement = DEFAULT_SITE_DISPLACEMENT;
		fprintf(stderr, "\nCannot find data on study site displacement. Taking default\n");
	}

	if ( GetFloat(list, "Weather station roughness", &WS_Roughness) == 0) {
		WS_Roughness = DEFAULT_WS_ROUGHNESS;
		fprintf(stderr, "\nCannot find data on weather station roughness. Taking default\n");
	}

	if ( GetFloat(list, "Weather station displacement", &WS_Displacement) == 0) {
		WS_Displacement = DEFAULT_WS_DISPLACEMENT;
		fprintf(stderr, "\nCannot find data on weather station displacement. Taking default\n");
	}

	if ( GetFloat(list, "Wind measurement height", &Measurement_Height) == 0) {
		Measurement_Height = DEFAULT_MEASUREMENT_HEIGHT;
		fprintf(stderr, "\nCannot find data on wind measurement height. Taking default\n");
	}

	DeleteRecord(list);

	return (number_of_periods);
}

void Get_Trees_Grid(void)
{
	FILE *fp;
	int i, j;
	char buf[20];

	if ((fp = fopen("trees.grd", "rt")) == NULL) {
		fprintf(stderr, "\nCannot open trees.grd input file.\n");
		exit(0);
	}
	else {
		printf("\nInput file 'trees.grd' opened");
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Columns);
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Rows);
		fgets(buf, 13, fp);
		fscanf(fp, "%ld\n", &xllcorner);
		fgets(buf, 13, fp);
		fscanf(fp, "%ld\n", &yllcorner);
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Cell_Width);
		fgets(buf, 13, fp);
		fscanf(fp, "%d", &nodata);

		if ( (site = malloc(sizeof(site_t)*Rows*Columns)) == NULL) {
			printf("Not enough memory to allocate buffer for %d x %d landscape\n",
				Rows, Columns);
			fclose(fp);
			printf("\nInput file 'trees.grd' closed\n");
			exit(1);  /* terminate program if out of memory */
		}
		Reset_Landscape();
		for (i = 0; i < Rows; i++)
			for (j = 0; j < Columns; j++)
				fscanf(fp, "%d", &site[i*Columns + j].source);
	}
	fclose(fp);
	printf("\nInput file 'trees.grd' closed\n");
}


void Get_Trees_XY(void)
{
	FILE *fp;
	char buf[20];
	long int canopy_area=0;
	int i, j, k=0, x, y, rad, initial_x, initial_y, last_x, last_y;
	int number_of_trees, actarea, noverlap;
	float distance, overlap=0.0;
	float* Trees_Array;

	if ((fp = fopen("treesxy.dat", "rt")) == NULL) {
		fprintf(stderr, "\nCannot open treesxy.dat input file.\n");
		exit(0);
	}
	else {
		printf("\nInput file 'treesxy.dat' opened");
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Columns);
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Rows);
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &Cell_Width);
		fgets(buf, 13, fp);
		fscanf(fp, "%d\n", &number_of_trees);
		fgets(buf, 20, fp);

		Trees_Array=(float*)malloc(sizeof(float)*TREE_ARRAY_PARAMETERS*number_of_trees);
		if (Trees_Array==NULL) {
			printf("Not enough memory to allocate buffer for %d trees\n",
				number_of_trees);
			fclose(fp);
			printf("\nInput file 'treesxy.dat' closed\n");
			exit(1);  /* terminate program if out of memory */
		}

		for (i = 0; i < TREE_ARRAY_PARAMETERS; i++)
			for (j = 0; j < number_of_trees; j++)
				Trees_Array[i*number_of_trees+j] = 0.0;

		if ( (site = malloc(sizeof(site_t)*Rows*Columns)) == NULL) {
			printf("Not enough memory to allocate buffer for %d x %d landscape\n",
				Rows, Columns);
			fclose(fp);
			printf("\nInput file 'treesxy.dat' closed\n");
			exit(1);  /* terminate program if out of memory */
		}

		Reset_Landscape();
		printf("\n\n Serial   X      Y    Radius");
		do {
			fscanf(fp, "%d", &k);
			printf("\n %4d   ", k);
			fscanf(fp, "%f", &Trees_Array[k-1]);
			printf("%4.0f   ", Trees_Array[k-1]);
			fscanf(fp, "%f", &Trees_Array[number_of_trees+k-1]);
			printf("%4.0f   ", Trees_Array[number_of_trees+k-1]);
			fscanf(fp, "%f", &Trees_Array[2*number_of_trees+k-1]);
			printf("%4.0f   ", Trees_Array[2*number_of_trees+k-1]);
		} while ( k<number_of_trees && !feof(fp) );

		fclose(fp);
		printf("\nInput file 'treesxy.dat' closed\n");
	}

	printf("\n\n");
	for (k = 0; k < number_of_trees; k++) {
		printf("\r Calculating canopy area... %3.0f%% done\r", 100.0*(k+1)/
		number_of_trees);
		actarea = 0;
		noverlap = 0;
		x = floor(0.5+Trees_Array[k]) - 1;
		y = floor(0.5+Trees_Array[number_of_trees+k]) - 1;
		/* 0,0 is the lower left corner */
		rad = floor(0.5+Trees_Array[2*number_of_trees+k]);
		initial_x = x - rad;
		if (initial_x < 0)
			initial_x = 0;
		initial_y = y - rad;
		if (initial_y < 0)
			initial_y = 0;
		last_x = x + rad;
		if (last_x > Columns)
			last_x = Columns;
		last_y = y + rad;
		if (last_y > Rows)
			last_y = Rows;
		for (i = initial_x; i < last_x; i++)
			for (j = initial_y; j < last_y; j++) {
				distance = sqrt( (x-i)*(x-i) + (y-j)*(y-j) );
				if (distance <= rad) {
					actarea++;
					if (!site[i*Columns + j].source) {
						noverlap++;
						canopy_area++;
						site[i*Columns + j].source = TRUE;
					}
					else
						overlap++;
				}
		}
	}
	printf("\n Total canopy area estimated as %ld m^2 (%.2f%% of the plot area)",
		canopy_area, 100.0*canopy_area / (Columns*Rows) );
	printf("\n Degree of canopy overlap: %.2f%%", 100.0*overlap/canopy_area );
}

int Get_This_Period_Data(char* period, float *MeanSeeRel, float *StdSeeRel,
		float *MeanLnWin, float *StdLnWin, float *MaxLnWin, float *MinLnWin)
{
	FILE *fp;
	int i, j;
	int DaysPeriod;
	char FileName[1000];
	Record *list;

	for (i = 0; i < WIND_SECTORS; i++)
		for (j = 0; j < WIND_ARRAY_PARAMETERS; j++)
			Wind_Array[i][j] = 0.0;

	sprintf(FileName, "%s.dat", period);

	list = Parse(FileName, '=');

	if (list==NULL) {
		fprintf(stderr, "Period Data File %s Open Error!\n", FileName);
		exit(1);
	}

	if ( GetInt(list, "Days in this period", &DaysPeriod) == 0) {
		fprintf(stderr, "\nCannot find data on the number of days in this period. Program terminated.\n");
		DeleteRecord(list);
		exit(1);  /* terminate program if critical data is missing */
	}

	if ( GetFloat(list, "Mean seed release", MeanSeeRel) == 0) {
		*MeanSeeRel = DEFAULT_MEAN_SEED_RELEASE;
		fprintf(stderr, "\nCannot find data on mean seed release. Taking default.\n");
	}

	if ( GetFloat(list, "Std seed release", StdSeeRel) == 0) {
		*StdSeeRel = DEFAULT_STD_SEED_RELEASE;
		fprintf(stderr, "\nCannot find data on std seed release. Taking default.\n");
	}

	if ( GetFloat(list, "Mean ln horizontal windspeed", MeanLnWin) == 0) {
		*MeanLnWin = DEFAULT_MEAN_LN_HORIZONTAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on mean ln horizontal windspeed. Taking default.\n");
	}

	if ( GetFloat(list, "Std ln horizontal windspeed", StdLnWin) == 0) {
		*StdLnWin = DEFAULT_STD_LN_HORIZONTAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on std ln horizontal windspeed. Taking default.\n");
	}

	if ( GetFloat(list, "Max ln horizontal windspeed", MaxLnWin) == 0) {
		*MaxLnWin = DEFAULT_MAX_LN_HORIZONTAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on max ln horizontal windspeed. Taking default.\n");
	}

	if ( GetFloat(list, "Min ln horizontal windspeed", MinLnWin) == 0) {
		*MinLnWin = DEFAULT_MIN_LN_HORIZONTAL_WINDSPEED;
		fprintf(stderr, "\nCannot find data on min ln horizontal windspeed. Taking default.\n");
	}

	DeleteRecord(list);

	if(!WindRandom) {
		sprintf(FileName, "%s.wnd", period);

		if ((fp = fopen(FileName, "rt")) == NULL) {
			fprintf(stderr, "\nCannot open period input file %s \n", FileName);
			exit(1);
		}
		else {
			printf("\nInput file '%s' opened", FileName);
			for (i = 0; i < WIND_SECTORS; i++)
				for (j = 0; j < WIND_ARRAY_PARAMETERS; j++)
					fscanf(fp, "%f", &Wind_Array[i][j]);
			fclose(fp);
			printf("\nInput file '%s' closed\n", FileName);
		}
	}

	return (DaysPeriod);
}

void Reset_Landscape(void)
{
	int i;

	for (i = 0; i < Columns*Rows; i++) {
		site[i].source = FALSE;
		site[i].this_period_seeds = 0;
		site[i].all_periods_seeds = 0;
	}
}

